#!/usr/bin/env /usr/bin/python
# -*- coding: ISO-8859-1 -*-

VERSION = "0.3.2"
MAXLOOP = 100

import ConfigParser 
import os
from optparse import OptionParser
import sys

class CustomError(Exception):
	def __init__(self,value):
		self.parameter=value
	def __str__(self):
		return repr(self.parameter)

class Host:
	def __init__(self, name, host, parent, options, ssh):
		self.name = name
		self.host = host
		self.parent = parent
		self.options = options
		self.ssh = ssh
	
	def __repr__(self):
		return self.name

class Config:
	def __init__(self, verbose = False):
		self.verbose = verbose
		self.__read()

	def __read(self):
		self.config = ConfigParser.ConfigParser()
		fname = open(os.path.expanduser( '~' ) + "/.sssh","r")
		self.config.readfp(fname)

		self.aliases = {}
		# build aliases list
		for section in self.config.sections():
			if self.config.has_option(section, 'alias'):
				for configAlias in self.config.get(section,'alias').split(','):
					configAlias = configAlias.strip()
					self.aliases[configAlias] = section

		fname.close()
	
	def getHost(self, hostName):
		try:
			host = self.config.get(hostName, "host")
		except:
			if self.aliases.has_key(hostName):
				host = self.aliases[hostName]
				hostName = host
				host2 = self.config.get(host, "host")
				host = host2
			else:
				host = hostName

		try:
			parent = self.config.get(hostName, "parent")
		except:
			parent = ""

		try:
			options = self.config.get(hostName, "options")
		except:
			try:
				options = self.config.get("main", "options");
			except:
				options = ""

		try:
			ssh = self.config.get(hostName, "ssh")
		except:
			try:
				ssh = self.config.get("main", "ssh")
			except:
				ssh = "ssh"
		
		return Host(hostName, host, parent, options, ssh)
	def getGraph(self):
		output = "digraph G {\n"
		sections = self.config.sections()
		sections.sort()
		for section in sections:
			if section != "main":
				output = output + section + " [];\n"

		for section in sections:
			if section != "main":
				parent = self.getHost(section).parent
				if parent != "":
					output = output + self.getHost(section).parent + "->" + section + ";\n"

		output = output + "}"
		return output

	def getPath(self, alias):
		end = False
		path = []
		path.append(self.getHost(alias))
		x = MAXLOOP
		

		while self.config.has_section(path[-1].parent):
			path.append(self.getHost(path[-1].parent))
			if x == 0:
				raise CustomError("There is a loop between those servers. Please, check the config file.")
			x = x -1 
		if self.verbose:
			prettyPath = "localhost"
			for i in reversed(range(len(path))):
				prettyPath = prettyPath + " -> " + path[i].name + " (" + path[i].host + ")"
				
			print "Calculated path: " + prettyPath
		return path

	def getCmdLine(self, path, mode, login, loginToAllHosts = False, command = ""):

		if login:
			if loginToAllHosts:
				x = 0
				for host in path:
					path[x].host = login + "@" + host.host
					x = x + 1
			else:
				path[0].host = login + "@" + path[-1].host
		
		path.reverse()
		first = True
		for host in path:
			if first:
				cmdLine = ""
				first = False
			else:
				cmdLine = cmdLine + " -t "
			cmdLine = cmdLine + host.ssh + " " + host.options + " " + host.host
		cmdLine = cmdLine + " \"" + command + "\""
		return cmdLine


def genConfig():
	print """
[main]
# Default ssh path
ssh = /usr/bin/ssh
# Use compression and quiet ssh by default
options = -qC

[firewall]
host = 192.168.0.1

[monitor]
parent = firewall
host = 192.168.1.12

[web1]
parent = firewall
host = 192.168.1.1

[web2]
parent = firewall
host = 192.168.1.2

# Connection to db servers will be done through monitor server
[database1]
host = 192.168.2.10
alias = db1
parent = monitor

[database2]
host = 192.168.2.11
alias = db2
parent = monitor

# sssh can also be used to connect to windows machines
[windows]
ssh = rdesktop
options = -5 -P -a 24 -g 1024x768 -u Administrator -p password
host = 192.168.0.12

"""
	sys.exit(-1)

def main(mode="ssh"):
	# define commandline options
	if mode == 'ssh':
		usage = "usage: %prog [options] [LOGIN@]host ['command']\n Connect to a ssh host following a defined route"
	elif mode == 'scp':
		usage = "usage: %prog [options] [[user@]host1:]file1 [...] [[user@]host2:]file2\n Copy a file to a ssh host following a defined route"

	parser = OptionParser(usage, version=VERSION)
	parser.add_option("-v", "--verbose", action="store_true", dest="verbose", default=False, help="verbose output")
	parser.add_option("-d", "--dry", action="store_true", dest="dry", default=False, help="Dry run. Doesn't execute ssh command.")
	parser.add_option("-u", "--output", action="store_true", dest="genConfig", default=False, help="Output configuration in the format used by the $HOME/.sssh file. Generate a new config file using: sssh -u > ~/.sssh")
	parser.add_option("-g", "--graph", action="store_true", dest="graphviz", default=False, help="Output hosts graph in graphviz format. Plot it using:     sssh -g | dot -Tpng > graph.png")
	parser.add_option("-l", "--login", action="store", dest="login", type="string", help="Use specified login to connect the last host")
	parser.add_option("-L", "", action="store_true", dest="loginToAllHosts", default=False, help="Use specified login in all hosts")

	(options, args) = parser.parse_args()

	# output sample config
	if options.genConfig:
		genConfig()
	
	try:
		config = Config(verbose = options.verbose)
	except IOError:
		parser.error("config file is missing. Create one whith: sssh -u > ~/.sssh")
		
	# output graph
	if options.graphviz:
		print config.getGraph()
		sys.exit(-1)

	# check if host exists in commandline
	if len(args) < 1:
		parser.error("incorrect number of arguments")
	
	# run
	hostlist = args[0].split('@')
	if len(hostlist) == 1:
		login = options.login
		host = hostlist[0]
	else:
		login = hostlist[0]
		host = hostlist[1]

	try:
		command = args[1]
		#command = ""
		#for arg in args[1:]:
		#	command = command + " " + arg
	except:
		command = ""

	try:
		cmdLine = config.getCmdLine(config.getPath(host), mode, login, options.loginToAllHosts, command)
	except CustomError, (instance):
		parser.error(instance)
		sys.exit(-1)

	if options.verbose:
		print "Executing: " + cmdLine
	if not options.dry:
		os.system(cmdLine)

